@@ -7,13 +7,19 @@
   DATA_MATRIX(C_hist);    // Total catch by year and fleet
   DATA_VECTOR(C_eq);      // Equilibrium catch by fleet
 
+  DATA_MATRIX(E_hist);    // Effort by year and fleet
+  DATA_VECTOR(E_eq);      // Equilibrium effort by fleet
+
+  DATA_STRING(condition); // Indicates whether the model will condition on effort or catch
+  DATA_INTEGER(nll_C);    // Indicates whether there is a likelihood for the catch if condition = "effort"
+
   DATA_MATRIX(I_hist);    // Index by year and survey
   DATA_MATRIX(sigma_I);   // Standard deviation of index by year and survey
 
-  DATA_ARRAY(CAA_hist);   // Catch-at-age proportions by year, age, fleet
+  DATA_ARRAY(CAA_hist);   // Catch-at-age re-weighted by year, age, fleet
   DATA_MATRIX(CAA_n);     // Annual samples in CAA by year and fleet
 
-  DATA_ARRAY(CAL_hist);   // Catch-at-length proportions by year, length_bin, fleet
+  DATA_ARRAY(CAL_hist);   // Catch-at-length re-weighted by year, length_bin, fleet
   DATA_MATRIX(CAL_n);     // Annual samples in CAL by year and fleet
 
   DATA_VECTOR(length_bin);// Vector of length bins
@@ -37,12 +43,15 @@
   DATA_MATRIX(LWT_C);     // LIkelihood weights for catch, CAA, CAL, ML, C_eq
   DATA_VECTOR(LWT_Index); // LIkelihood weights for the index
 
-  DATA_VECTOR(est_early_rec_dev);
-  DATA_VECTOR(est_rec_dev); // Indicator of whether rec_dev is estimated in model or fixed at zero
+  DATA_SCALAR(max_F);
+
+  DATA_IVECTOR(est_early_rec_dev);
+  DATA_IVECTOR(est_rec_dev); // Indicator whether to estimate rec_dev
 
   PARAMETER(log_R0);
   PARAMETER(transformed_h);
   PARAMETER_MATRIX(vul_par);            // Matrix of vul_par
+  PARAMETER_VECTOR(log_q_effort);
   //PARAMETER_VECTOR(log_mean_F);
   PARAMETER_MATRIX(log_F);
   PARAMETER_VECTOR(log_F_equilibrium);  // Equilibrium U by fleet
@@ -65,70 +74,70 @@
   h += 0.2;
 
   Type tau = exp(log_tau);
-  vector<Type> sigma_mlen(nfleet);
-  vector<Type> F_equilibrium(nfleet);
-  F_equilibrium.setZero();
-  for(int ff=0;ff<nfleet;ff++) {
-    sigma_mlen(ff) = exp(log_sigma_mlen(ff));
-    if(C_eq(ff)>0) F_equilibrium(ff) = exp(log_F_equilibrium(ff));
-  }
 
+  // Vulnerability (length-based) and F parameters
   Type penalty = 0;
   Type prior = 0.;
 
-  // Vulnerability (length-based)
-  vector<vector<Type> > vul(nfleet);
-  vul = calc_vul(vul_par, vul_type, nlbin, length_bin, prior);
+  matrix<Type> vul = calc_vul(vul_par, vul_type, nlbin, length_bin, prior);
+
+  vector<Type> q_effort(nfleet);
+  vector<Type> sigma_mlen(nfleet);
+  vector<Type> F_equilibrium(nfleet);
+  F_equilibrium.setZero();
 
   matrix<Type> F(n_y,nfleet);
-  for(int ff=0;ff<nfleet;ff++) {
-    //for(int y=0;y<n_y;y++) F(y,ff) = exp(log_mean_F(ff) + log_F_dev(y,ff));
-    for(int y=0;y<n_y;y++) F(y,ff) = exp(log_F(y,ff));
-  }
-
   array<Type> Z_total(n_y, max_age, nlbin);
 
-  ////// Equilibrium reference points and per-recruit quantities
-  matrix<Type> NPR_unfished(max_age, nlbin);
-  matrix<Type> ALK_unfished(max_age, nlbin);
-  ALK_unfished = generate_ALK(length_bin, len_age, CV_LAA, max_age, nlbin, bin_width, n_y-1);
-  NPR_unfished = calc_NPR0(nlbin, M, max_age, ALK_unfished, n_y-1);
+  for(int ff=0;ff<nfleet;ff++) {
+    sigma_mlen(ff) = exp(log_sigma_mlen(ff));
+    q_effort(ff) = exp(log_q_effort(ff));
+    if(condition == "catch" && C_eq(ff)>0) F_equilibrium(ff) = exp(log_F_equilibrium(ff));
+    if(condition == "effort" && E_eq(ff)>0) F_equilibrium(ff) = q_effort(ff) * E_eq(ff);
 
-  Type EPR0 = sum_EPR(NPR_unfished, wt_at_len, mat, max_age, nlbin, n_y-1);
+    for(int y=0;y<n_y;y++) {
+      if(condition == "catch") {
+        F(y,ff) = CppAD::CondExpLt(max_F - exp(log_F(y)), Type(0), max_F - posfun(max_F - exp(log_F(y,ff)), Type(0), penalty),
+          exp(log_F(y,ff)));
+      } else {
+        F(y,ff) = CppAD::CondExpLt(max_F - q_effort(ff) * E_hist(y,ff), Type(0), max_F - posfun(max_F - q_effort(ff) * E_hist(y,ff), Type(0), penalty),
+          q_effort(ff) * E_hist(y,ff));
+      }
+    }
+  }
 
-  Type B0 = R0 * sum_BPR(NPR_unfished, wt_at_len);
-  Type N0 = R0 * NPR_unfished.sum();
-  Type E0 = R0 * EPR0;
-  //Type VB0 = R0 * sum_VBPR(NPR_unfished, wt_at_len, vul, max_age, nlbin);
 
-  matrix<Type> NPR_unfished_year1(max_age, nlbin);
-  matrix<Type> ALK_unfished_year1(max_age, nlbin);
-  ALK_unfished_year1 = generate_ALK(length_bin, len_age, CV_LAA, max_age, nlbin, bin_width, 0);
-  NPR_unfished_year1 = calc_NPR0(nlbin, M, max_age, ALK_unfished, 0);
+  ////// Equilibrium reference points and per-recruit quantities - calculate annually
+  vector<Type> EPR0(n_y);
+  vector<Type> E0(n_y);
+  vector<Type> B0(n_y);
+  vector<Type> N0(n_y);
 
-  Type EPR0_year1 = sum_EPR(NPR_unfished_year1, wt_at_len, mat, max_age, nlbin, 0);
+  vector<Type> Arec(n_y);
+  vector<Type> Brec(n_y);
 
-  //Type B0 = R0 * sum_BPR(NPR_unfished, wt_at_len);
-  //Type N0 = R0 * NPR_unfished.sum();
-  Type E0_year1 = R0 * EPR0_year1;
+  for(int y=0;y<n_y;y++) {
+    matrix<Type> ALK_unfished = generate_ALK(length_bin, len_age, CV_LAA, max_age, nlbin, bin_width, y);
+    matrix<Type> NPR_unfished = calc_NPR0(nlbin, M, max_age, ALK_unfished, y);
 
-  Type Arec;
-  Type Brec;
+    EPR0(y) = sum_EPR(NPR_unfished, wt_at_len, mat, max_age, nlbin, y);
+    E0(y) = R0 * EPR0(y);
+    B0(y) = R0 * sum_BPR(NPR_unfished, wt_at_len);
+    N0(y) = R0 * NPR_unfished.sum();
 
-  if(SR_type == "BH") {
-    Arec = 4 *h;
-    Arec /= 1-h;
-    Arec /= EPR0;
-    Brec = 5*h - 1;
-    Brec /= (1-h) * E0;
-  } else {
-    Arec = pow(5*h, 1.25);
-    Arec /= EPR0;
-    Brec = 1.25;
-    Brec *= log(5*h);
-    Brec /= E0;
+    if(SR_type == "BH") {
+      Arec(y) = 4 *h;
+      Arec(y) /= 1-h;
+      Brec(y) = 5*h - 1;
+      Brec(y) /= (1-h) * E0(y);
+    } else {
+      Arec(y) = pow(5*h, 1.25);
+      Brec(y) = 1.25;
+      Brec(y) *= log(5*h);
+      Brec(y) /= E0(y);
+    }
+    Arec(y) /= EPR0(y);
   }
-  Type CR = Arec * EPR0;
 
   ////// During time series year = 1, 2, ..., n_y
   vector<matrix<Type> > ALK(n_y+1);
@@ -166,58 +175,53 @@
   E.setZero();
 
   // Equilibrium quantities (leading into first year of model)
-  matrix<Type> NPR_equilibrium(max_age, nlbin);
-  matrix<Type> ALK_equilibrium(max_age, nlbin);
-  ALK_equilibrium = generate_ALK(length_bin, len_age, CV_LAA, max_age, nlbin, bin_width, 0);
-  NPR_equilibrium = calc_NPR(F_equilibrium, vul, nfleet, nlbin, M, max_age, ALK_equilibrium, 0);
+  matrix<Type> ALK_equilibrium = generate_ALK(length_bin, len_age, CV_LAA, max_age, nlbin, bin_width, 0);
+  matrix<Type> NPR_equilibrium = calc_NPR(F_equilibrium, vul, nfleet, nlbin, M, max_age, ALK_equilibrium, 0);
 
   Type EPR_eq = sum_EPR(NPR_equilibrium, wt_at_len, mat, max_age, nlbin, 0);
   Type R_eq;
 
   if(SR_type == "BH") {
-    R_eq = Arec * EPR_eq - 1;
+    R_eq = Arec(0) * EPR_eq - 1;
   } else {
-    R_eq = log(Arec * EPR_eq);
+    R_eq = log(Arec(0) * EPR_eq);
   }
-  R_eq /= Brec * EPR_eq;
+  R_eq /= Brec(0) * EPR_eq;
 
   R(0) = R_eq;
-  if(!R_IsNA(asDouble(est_rec_dev(0)))) {
-    R(0) *= exp(log_rec_dev(0) - 0.5 * pow(tau, 2));
+  if(est_rec_dev(0)) {
+    R(0) *= exp(log_rec_dev(0) - 0.5 * tau * tau);
   }
 
   ALK(0) = generate_ALK(length_bin, len_age, CV_LAA, max_age, nlbin, bin_width, 0);
   for(int a=0;a<max_age;a++) {
     if(a == 0) {
-      N(0,a) = R(0) * get_N_at_age(NPR_equilibrium, nlbin, a);
+      N(0,a) = R(0) * NPR_equilibrium.row(a).sum();
     } else {
       R_early(a-1) = R_eq;
-      if(!R_IsNA(asDouble(est_early_rec_dev(a-1)))) {
-        R_early(a-1) *= exp(log_early_rec_dev(a-1) - 0.5 * pow(tau, 2));
-      }
-      N(0,a) = R_early(a-1) * get_N_at_age(NPR_equilibrium, nlbin, a);
+      if(est_early_rec_dev(a-1)) R_early(a-1) *= exp(log_early_rec_dev(a-1) - 0.5 * tau * tau);
+      N(0,a) = R_early(a-1) * NPR_equilibrium.row(a).sum();
     }
     for(int len=0;len<nlbin;len++) {
       N_full(0,a,len) = N(0,a) * ALK(0)(a,len);
-      for(int ff=0;ff<nfleet;ff++) VB(0,ff) += N_full(0,a,len) * wt_at_len(len) * vul(ff)(len);
+      for(int ff=0;ff<nfleet;ff++) VB(0,ff) += N_full(0,a,len) * wt_at_len(len) * vul(len,ff);
       B(0) += N_full(0,a,len) * wt_at_len(len);
       E(0) += N_full(0,a,len) * wt_at_len(len) * mat(0,a);
     }
   }
 
-  vector<Type> C_eq_pred(nfleet);
-  C_eq_pred = calc_C_eq(F_equilibrium, N_full, vul, M, wt_at_len, nlbin, nfleet, max_age, 0);
+  vector<Type> C_eq_pred = calc_C_eq(F_equilibrium, N_full, vul, M, wt_at_len, nlbin, nfleet, max_age, 0);
 
   // Loop over all other years
   for(int y=0;y<n_y;y++) {
     if(SR_type == "BH") {
-      R(y+1) = BH_SR(E(y), h, R0, E0);
+      R(y+1) = BH_SR(E(y), h, R0, E0(y));
     } else {
-      R(y+1) = Ricker_SR(E(y), h, R0, E0);
+      R(y+1) = Ricker_SR(E(y), h, R0, E0(y));
     }
 
     if(y<n_y-1) {
-      if(!R_IsNA(asDouble(est_rec_dev(y)))) R(y+1) *= exp(log_rec_dev(y+1) - 0.5 * pow(tau, 2));
+      if(est_rec_dev(y+1)) R(y+1) *= exp(log_rec_dev(y+1) - 0.5 * tau * tau);
     }
     N(y+1,0) = R(y+1);
 
@@ -227,15 +231,12 @@
 
     for(int a=0;a<max_age;a++) {
       for(int len=0;len<nlbin;len++) {
-        Z_total(y,a,len) = M(y,a);
-        for(int ff=0;ff<nfleet;ff++) Z_total(y,a,len) += vul(ff)(len) * F(y,ff);
-
+        Z_total(y,a,len) = M(y,a) + (vul.row(len) * F.row(y)).sum();
         Type mean_N = N_full(y,a,len) * (1 - exp(-Z_total(y,a,len)));
         mean_N /= Z_total(y,a,len);
 
         for(int ff=0;ff<nfleet;ff++) {
-          Type Cat_yalf = vul(ff)(len) * F(y,ff) * mean_N;
-
+          Type Cat_yalf = vul(len,ff) * F(y,ff) * mean_N;
           CAApred(y,a,ff) += Cat_yalf;
           CALpred(y,len,ff) += Cat_yalf;
           Cat(y,a,len) += Cat_yalf;
@@ -250,7 +251,7 @@
 
       for(int len=0;len<nlbin;len++) {
         N_full(y+1,a,len) = N(y+1,a) * ALK(y+1)(a,len);
-        for(int ff=0;ff<nfleet;ff++) VB(y+1,ff) += vul(ff)(len) * N_full(y+1,a,len) * wt_at_len(len);
+        for(int ff=0;ff<nfleet;ff++) VB(y+1,ff) += vul(len,ff) * N_full(y+1,a,len) * wt_at_len(len);
         B(y+1) += N_full(y+1,a,len) * wt_at_len(len);
         E(y+1) += N_full(y+1,a,len) * wt_at_len(len) * mat(x,a);
       }
@@ -258,6 +259,7 @@
     for(int ff=0;ff<nfleet;ff++) mlen_pred(y,ff) /= CN(y,ff);
   }
 
+
   // Calculate nuisance parameters and likelihood
   vector<Type> q(nsurvey);
   for(int sur=0;sur<nsurvey;sur++) {
@@ -284,7 +286,7 @@
   nll_CAA.setZero();
   nll_CAL.setZero();
   nll_ML.setZero();
-  //nll_Ceq.setZero();
+  nll_Ceq.setZero();
 
   for(int sur=0;sur<nsurvey;sur++) {
     for(int y=0;y<n_y;y++) {
@@ -295,25 +297,27 @@
 
   for(int ff=0;ff<nfleet;ff++) {
     for(int y=0;y<n_y;y++) {
-      if(C_hist(y,ff)>0) {
+      if(C_hist(y,ff)>0 || E_hist(y,ff)>0) {
 
-        nll_Catch(ff) -= dnorm(log(C_hist(y,ff)), log(Cpred(y,ff)), Type(0.01), true);
+        if(condition == "catch" || nll_C) nll_Catch(ff) -= dnorm(log(C_hist(y,ff)), log(Cpred(y,ff)), Type(0.01), true);
 
         if(!R_IsNA(asDouble(CAA_n(y,ff))) && CAA_n(y,ff) > 0) {
           vector<Type> loglike_CAAobs(max_age);
           vector<Type> loglike_CAApred(max_age);
-          for(int a=0;a<max_age;a++) loglike_CAApred(a) = CAApred(y,a,ff)/CN(y,ff);
-
-          for(int a=0;a<max_age;a++) loglike_CAAobs(a) = CppAD::CondExpLt(CAA_hist(y,a,ff), Type(1e-8), Type(1e-8), CAA_hist(y,a,ff)) * CAA_n(y,ff);
+          for(int a=0;a<max_age;a++) {
+            loglike_CAApred(a) = CAApred(y,a,ff)/CN(y,ff);
+            loglike_CAAobs(a) = CAA_hist(y,a,ff);
+          }
           nll_CAA(ff) -= dmultinom(loglike_CAAobs, loglike_CAApred, true);
         }
 
         if(!R_IsNA(asDouble(CAL_n(y,ff))) && CAL_n(y,ff) > 0) {
           vector<Type> loglike_CALobs(nlbin);
           vector<Type> loglike_CALpred(nlbin);
-          for(int len=0;len<nlbin;len++) loglike_CALpred(len) = CALpred(y,len,ff)/CN(y,ff);
-
-          for(int len=0;len<nlbin;len++) loglike_CALobs(len) = CppAD::CondExpLt(CAL_hist(y,len,ff), Type(1e-8), Type(1e-8), CAL_hist(y,len,ff)) * CAL_n(y,ff);
+          for(int len=0;len<nlbin;len++) {
+            loglike_CALpred(len) = CALpred(y,len,ff)/CN(y,ff);
+            loglike_CALobs(len) = CAL_hist(y,len,ff);
+          }
           nll_CAL(ff) -= dmultinom(loglike_CALobs, loglike_CALpred, true);
         }
 
@@ -327,15 +331,16 @@
     nll_CAA(ff) *= LWT_C(ff,1);
     nll_CAL(ff) *= LWT_C(ff,2);
     nll_ML(ff) *= LWT_C(ff,3);
-    if(C_eq(ff) > 0) nll_Ceq(ff) = LWT_C(ff,4) * dnorm(log(C_eq(ff)), log(C_eq_pred(ff)), Type(0.01), true);
+
+    if(condition == "catch" && C_eq(ff) > 0) nll_Ceq(ff) = LWT_C(ff,4) * dnorm(log(C_eq(ff)), log(C_eq_pred(ff)), Type(0.01), true);
 
   }
 
   for(int y=0;y<n_y;y++) {
-    if(!R_IsNA(asDouble(est_rec_dev(y)))) nll_log_rec_dev -= dnorm(log_rec_dev(y), Type(0), tau, true);
+    if(est_rec_dev(y)) nll_log_rec_dev -= dnorm(log_rec_dev(y), Type(0), tau, true);
   }
   for(int a=0;a<max_age-1;a++) {
-    if(!R_IsNA(asDouble(est_early_rec_dev(a)))) nll_log_rec_dev -= dnorm(log_early_rec_dev(a), Type(0), tau, true);
+    if(est_early_rec_dev(a)) nll_log_rec_dev -= dnorm(log_early_rec_dev(a), Type(0), tau, true);
   }
 
   Type nll = nll_Catch.sum() + nll_Index.sum();
@@ -346,6 +351,7 @@
   ADREPORT(R0);
   ADREPORT(h);
   ADREPORT(tau);
+  ADREPORT(q_effort);
   ADREPORT(q);
 
   REPORT(M);
@@ -354,6 +360,7 @@
   REPORT(log_R0);
   REPORT(transformed_h);
   REPORT(vul_par);
+  REPORT(log_q_effort);
   //REPORT(log_mean_F);
   REPORT(log_F);
   REPORT(log_F_equilibrium);
@@ -372,21 +379,13 @@
   REPORT(F);
   REPORT(Z_total);
 
-  REPORT(NPR_unfished);
-  REPORT(ALK_unfished);
   REPORT(EPR0);
   REPORT(B0);
   REPORT(E0);
   REPORT(N0);
 
-  REPORT(NPR_unfished_year1);
-  REPORT(ALK_unfished_year1);
-  REPORT(EPR0_year1);
-  REPORT(E0_year1);
-
   REPORT(Arec);
   REPORT(Brec);
-  REPORT(CR);
 
   REPORT(N_full);
   REPORT(ALK);
@@ -418,6 +417,7 @@
   REPORT(nll_CAL);
   REPORT(nll_ML);
   REPORT(nll_Ceq);
+  REPORT(nll_log_rec_dev);
 
   REPORT(nll);
   REPORT(penalty);
